package com.egommerce.apigateway;

import com.google.inject.Guice;
import com.google.inject.Injector;
import com.netflix.zuul.netty.server.BaseServerStartup;
import com.netflix.zuul.netty.server.Server;

import java.util.concurrent.TimeUnit;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

// import io.netty.handler.codec.http.DefaultHttpRequest;
import com.netflix.zuul.filters.endpoint.ProxyEndpoint;

public class Bootstrap {
    private static final Logger logger = LoggerFactory.getLogger(Bootstrap.class);

    public static void main(String[] args) {
        new Bootstrap().start();
    }

    public void start() {
        long startNanos = System.nanoTime();
        logger.info("###############");
        logger.info("Starting up...");
        int exitCode = 0;

        Server server = null;

        try {
            Injector injector = Guice.createInjector(new ZuulApiGwModule());
            BaseServerStartup serverStartup = injector.getInstance(BaseServerStartup.class);
            server = serverStartup.server();
            server.start();

            long startupDuration = System.nanoTime() - startNanos;
            logger.info("Finished startup. Duration = {}s", TimeUnit.NANOSECONDS.toSeconds(startupDuration));
            server.awaitTermination();
        } catch (Throwable t) {
            t.printStackTrace();
            logger.error("Initialization failed. Forcing shutdown now...");
            exitCode = 1;
        } finally {
            if (server != null) {
                server.stop();
            }
            logger.info("Shutdown completed. Quiting.");
            logger.info("###############");

            System.exit(exitCode);
        }
    }
}
