package com.egommerce.apigateway;

// import com.egommerce.apigateway.config.ApiGwServerSslConfig;
import com.egommerce.apigateway.module.ApiGwEurekaModule;
import com.egommerce.apigateway.module.ApiGwFiltersModule;
import com.google.inject.AbstractModule;
import com.netflix.config.ConfigurationManager;
import com.netflix.discovery.AbstractDiscoveryClientOptionalArgs;
import com.netflix.discovery.DiscoveryClient;
import com.netflix.discovery.shared.resolver.EndpointRandomizer;
import com.netflix.discovery.shared.resolver.ResolverUtils;
import com.netflix.netty.common.accesslog.AccessLogPublisher;
import com.netflix.netty.common.ssl.ServerSslConfig;
import com.netflix.netty.common.status.ServerStatusManager;
import com.netflix.spectator.api.DefaultRegistry;
import com.netflix.spectator.api.Registry;
import com.netflix.zuul.BasicRequestCompleteHandler;
import com.netflix.zuul.DynamicCodeCompiler;
import com.netflix.zuul.DynamicFilterLoader;
import com.netflix.zuul.FilterFileManager;
import com.netflix.zuul.FilterLoader;
import com.netflix.zuul.RequestCompleteHandler;
import com.netflix.zuul.context.SessionContextDecorator;
import com.netflix.zuul.context.ZuulSessionContextDecorator;
import com.netflix.zuul.filters.FilterRegistry;
import com.netflix.zuul.filters.MutableFilterRegistry;
import com.netflix.zuul.groovy.GroovyCompiler;
import com.netflix.zuul.groovy.GroovyFileFilter;
import com.netflix.zuul.netty.server.BaseServerStartup;
import com.netflix.zuul.netty.server.ClientRequestReceiver;
import com.netflix.zuul.origins.BasicNettyOriginManager;
import com.netflix.zuul.origins.OriginManager;
import com.netflix.zuul.stats.BasicRequestMetricsPublisher;
import com.netflix.zuul.stats.RequestMetricsPublisher;
import java.io.FilenameFilter;
import org.apache.commons.configuration.AbstractConfiguration;
// import org.slf4j.Logger;
// import org.slf4j.LoggerFactory;

public class ZuulApiGwModule extends AbstractModule {
    // private static final Logger log = LoggerFactory.getLogger(ZuulApiGwModule.class);

    @Override
    protected void configure() {
        try {
          ConfigurationManager.loadCascadedPropertiesFromResources("application");
        } catch (Exception ex) {
          throw new RuntimeException("Error loading configuration: " + ex.getMessage(), ex);
        }

        bind(AbstractConfiguration.class).toInstance(ConfigurationManager.getConfigInstance());
        bind(DynamicCodeCompiler.class).to(GroovyCompiler.class);
        bind(FilenameFilter.class).to(GroovyFileFilter.class);

        // bind(ServerSslConfig.class).to(ApiGwServerSslConfig.class);

        // zuul eureka module
        bind(EndpointRandomizer.class).toInstance(ResolverUtils::randomize);
        install(new ApiGwEurekaModule());

        // server specific bindings
        bind(BaseServerStartup.class).to(ApiGwServerStartup.class);
        // use provided basic netty origin manager
        bind(OriginManager.class).to(BasicNettyOriginManager.class);
        
        // general server bindings
        bind(ServerStatusManager.class); // health/discovery status
        bind(SessionContextDecorator.class).to(ZuulSessionContextDecorator.class); // decorate new sessions when requests come in
        bind(Registry.class).to(DefaultRegistry.class); // atlas metrics registry
        bind(RequestCompleteHandler.class).to(BasicRequestCompleteHandler.class); // metrics post-request completion
        bind(AbstractDiscoveryClientOptionalArgs.class).to(DiscoveryClient.DiscoveryClientOptionalArgs.class); // discovery client
        bind(RequestMetricsPublisher.class).to(BasicRequestMetricsPublisher.class); // timings publisher
        
        // zuul filter loading
        install(new ApiGwFiltersModule());
        bind(FilterLoader.class).to(DynamicFilterLoader.class);
        bind(FilterRegistry.class).to(MutableFilterRegistry.class);
        bind(FilterFileManager.class).asEagerSingleton();

        
        // access logger, including request ID generator
        bind(AccessLogPublisher.class).toInstance(new AccessLogPublisher("ACCESS",
            (channel, httpRequest) -> ClientRequestReceiver.getRequestFromChannel(channel).getContext().getUUID())
        );
    }
}
